#!/bin/sh
#
# P5 installation script.
#
# See the file "license.txt" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# Rcsid: @(#)$Id: install.sh,v 1.78 2017/07/24 16:51:17 zv Exp $
# ----------------------------------------------------------------------
#

usage() {
    echo "Usage: $0 [server] [port]"
    exit 1
}

#
# Synology and QNAP handle library linking and start/stop differently
# and we have to know if we're on Synology/QNAP or regular Linux
#

#
# Init & Detection for Synology
#

SYNOLOGY=0
if test -s /bin/get_key_value && test -s /etc/synoinfo.conf; then
    case `/bin/get_key_value /etc/synoinfo.conf unique` in
      synology*)
	    SYNOLOGY=1
	;;
    esac
fi

#
# Init & Detection for QNAP
#

QNAP=0
if test -s /etc/model.conf; then
    VENDOR=`head /etc/model.conf | grep VENDOR`
    if test "$VENDOR" = "VENDOR = QNAP"; then
        QNAP=1
    fi
fi

# 
# Test for root user.
#

case `uname -s` in
  SunOS)
    id=/usr/xpg4/bin/id
    ;;
  *)
    id=/usr/bin/id
    ;;
esac
if test `$id -u` -ne 0; then
    echo "Installing P5 can only be done as user root!" >&2
    exit 1
fi

#
# Get the home directory
#

dir=`dirname "$0"`
cwd=`pwd`;cd "$dir"
LEXXDIR=`pwd`;export LEXXDIR
cd "$cwd"

#
# See if something left do before we install
#

if test -x "$LEXXDIR/.preinstall"; then
    /bin/sh "$LEXXDIR/.preinstall"
    if test $? -ne 0; then
        exit 1
    fi
fi

#
# Assure config directory is present
#

if test ! -d "$LEXXDIR/config" -a ! -h "$LEXXDIR/config"; then
    rm -f "$LEXXDIR/config"
    mkdir "$LEXXDIR/config"
fi

#
# Get servername, port
#

# Synology uses port 20000 since 8000 is often already in use

if test "$SYNOLOGY" -eq "1"; then
	PORT=20000
else
	PORT=8000
fi

if test $# -gt 1; then
    SERVERNAME=$1
    shift
    srvdir="$LEXXDIR/servers/$SERVERNAME"
    if test "$SERVERNAME" != "lexxsrv" -a ! -h "$srvdir" -a ! -d "$srvdir"; then
        rm -f "$srvdir"
        ln -s "lexxsrv" "$srvdir"
    fi
else
    SERVERNAME="lexxsrv"
fi

#
# Fix link to the correct binaries
#

case `uname -s` in
    Linux)	
	    if test "$SYNOLOGY" -eq "0"; then
	
            init=/sbin/init
            if test -h $init; then
                init=`readlink -f $init`
            fi
            file $init 2>/dev/null | grep 32-bit >/dev/null
            if test $? -eq 0; then
                PBIN=binaries/Linux/unknown
            else
                PBIN=binaries/Linux/unknown/64
            fi
		# Synology uses different method to check if we are 32 or 64 bit and link accordingly	
		elif test "$SYNOLOGY" -eq "1"; then
		    init_pid=1
	        if test ! -e /proc/$init_pid/maps; then
    	        echo "Process is not found"
	        else
    	        case `awk </proc/$init_pid/maps -- 'END { print substr($1, 0, 9); }'` in
    	   	      *-)
	   	            # 32-bit
		            PBIN=binaries/Linux/unknown
		            ;;
    		      *[0-9A-Fa-f])
	   	            # 64-bit
		            PBIN=binaries/Linux/unknown/64
		            ;;
    		      *)
		            echo "Insufficient permissions."
		            ;;
    	        esac
 	        fi
		fi
        ;;
    FreeBSD)
        PBIN=binaries/FreeBSD/amd64/`uname -r | awk -F. '{print $1}'`
        ;;
    Darwin)
        PBIN=binaries/Darwin/`uname -m`
        ;;
    SunOS)
        PBIN=binaries/SunOS/`uname -p`/64
        if test ! -d "$LEXXDIR/$PBIN"; then
            echo "Unsupported platform: `uname -s`"
            exit 130
        fi
        ;;
    *)
        echo "Unsupported platform: `uname -s`"
        exit 130
        ;;
esac

#
# Fix link to the correct binaries.
#

if test ! -r "$LEXXDIR/bin" -o -L "$LEXXDIR/bin"; then
    cd "$LEXXDIR"
    rm -f bin
    ln -s "$PBIN" bin
    cd "$cwd"
fi

done="n"
Slev=99
Klev=01

case `uname -s` in
  Linux)
    for dir in /etc/rc.d /sbin/init.d; do
        if test -d $dir; then
            rc=$dir
            break
        fi
    done
    ;;
  SunOS|*)
    rc=/etc
    ;;
esac

while test "$done" != "y"; do
    ret=0
    if test $# -ge 1; then
        line="1"
        done="y"
    else
    cat <<EOF1

        ArchiWare P5 Installation Menu
        ------------------------------

         1)      Install or configure the application server
EOF1
    case `uname -s` in
        SunOS)
                cat <<EOF2
        10)      Install the media changer driver
EOF2
    esac
    cat <<EOF3

        99)      Quit

EOF3

    printf "Your choice: "
    read line
    fi

    case "$line" in
      1)
        if test $# -ge 1; then
            "$LEXXDIR/etc/addsrv" $SERVERNAME $1
            PORT=$1
        else
            "$LEXXDIR/etc/addsrv" $SERVERNAME
        fi
        ret=$?
        if test $ret -eq 100; then
            if test $# -eq 0; then
                echo "You must restart server for any changes to take effect."
                echo ""
                printf "Press return key to continue..."
                read dummy
            fi
            ret=0
        fi
        case "`uname -s`" in
          Darwin)
            rm -f /var/log/awlogs
            ln -s "$LEXXDIR/log" /var/log/awlogs
            rc=/Library/StartupItems/PresStore
            if test -d $rc; then
                rm -rf $rc
            fi
            pl=/Library/LaunchDaemons/com.archiware.presstore.$SERVERNAME.plist
            sed -e "s|HOME|$LEXXDIR|g"    \
                -e "s|SERV|$SERVERNAME|g" \
                -e "s|USER|root|g"        \
                -e "s|PORT|$PORT|g" < "$LEXXDIR/etc/LaunchDaemon.plist" > $pl
            chmod 0644 $pl
            oldpp="/Library/PreferencePanes/PresSTORE.prefPane"
            if test -d "$oldpp"; then
                rm -rf "$oldpp"
            fi
            prefp="$LEXXDIR/etc/Archiware P5.prefPane"
            if test -d "$prefp"; then
                ditto "$prefp" "/Library/PreferencePanes/Archiware P5.prefPane"
            fi
            ;;
          Linux|SunOS)
		    # QNAP & Synology handle start/stop differently so we skip this
            if test "$SYNOLOGY" -eq "0" && test "$QNAP" -eq "0"; then
                echo ""
                echo "Arranging for server to start/stop at power cycle"
                sf=/etc/init.d/lexx
                echo "#!/bin/sh" > $sf
				echo "### BEGIN INIT INFO" >> $sf
                echo "# Provides: lexx" >> $sf
                echo "# Required-Start:" >> $sf
                echo "# Required-Stop:" >> $sf
                echo "# Default-Start:      2 3 4 5" >> $sf
                echo "# Default-Stop:       0 1 6" >> $sf
                echo "# Short-Description: lexxserver" >> $sf
                echo "### END INIT INFO" >> $sf
                echo "\"$LEXXDIR/etc/rc.lexx\" \$*" >> $sf
                chmod 0755 $sf
                if test -x /usr/sbin/update-rc.d; then
                    /usr/sbin/update-rc.d lexx defaults $Slev $Klev
                else
                    printf "  start: added S${Slev}lexx to $rc/rcX.d, X=("
                    for i in 3 4 5; do
                        if test -d $rc/rc$i.d; then
                            rm -f $rc/rc$i.d/S*lexx
                            ln -s $sf $rc/rc$i.d/S${Slev}lexx
                            printf " $i"
                        fi
                    done
                    echo " )."
                    printf "  stop:  added K${Klev}lexx to $rc/rcX.d, X=("
                    for i in 0 1 2 6 S; do
                        if test -d $rc/rc$i.d; then
                            rm -f $rc/rc$i.d/K*lexx
                            ln -s $sf $rc/rc$i.d/K${Klev}lexx
                            printf " $i"
                        fi
                    done
                    echo " )."
                fi
            fi
            ;;
          FreeBSD)
            echo ""
            echo "Arranging for server to start/stop at power cycle"
            sf=/etc/rc.d/lexx
            echo "#!/bin/sh" > $sf
            echo "# REQUIRE: NETWORKING FILESYSTEMS" >> $sf
            echo ". /etc/rc.subr" >> $sf
            echo "name=\"lexx\"" >> $sf
            echo "rcvar=\"lexx_enable\"" >> $sf
            echo "start_cmd=\"lexx_start\"" >> $sf
            echo "stop_cmd=\"lexx_stop\"" >> $sf
            echo "lexx_start()" >> $sf
            echo "{" >> $sf
            echo "\"$LEXXDIR/etc/rc.lexx\" start" >> $sf
            echo "}" >> $sf
            echo "lexx_stop()" >> $sf
            echo "{" >> $sf
            echo "\"$LEXXDIR/etc/rc.lexx\" stop" >> $sf
            echo "}" >> $sf
            echo "load_rc_config lexx" >> $sf
            echo "run_rc_command \"\$1\"" >> $sf
            echo "Added $sf"
            if test -z "`grep lexx_enable /etc/rc.conf`"; then
                echo "lexx_enable=\"YES\"" >> /etc/rc.conf
                echo "Added lexx_enable=\"YES\" to /etc/rc.conf"
            fi
            chown root:wheel $sf
            chmod 0755 $sf
			# check if we are running on FREENAS*
			# if yes we need to copy files to /conf/base/etc folder to ensure that it survives restart
            case "`uname -i`" in
              FREENAS*)
                if test -d "/conf/base/etc/rc.d"; then
                    cp /etc/rc.d/lexx /conf/base/etc/rc.d/lexx
                    chown root:wheel /conf/base/etc/rc.d/lexx
                    chmod 0755 /conf/base/etc/rc.d/lexx
                    echo "lexx_enable=\"YES\"" >> /conf/base/etc/rc.conf
                fi
              ;;
            esac											  
            ;;			
        esac
        case "`uname -s`" in
          Linux|FreeBSD)
            pam=/etc/pam.d/nsd
            echo "# P5 installation: `/bin/date`" > $pam
			# QNAP has to have full path to pam_unix.so
			if test "$QNAP" -eq "1"; then
			    echo "auth  required        /usr/local/lib/pam_unix.so" >> $pam
                echo "account       required        /usr/local/lib/pam_unix.so" >> $pam
            else
			    echo "auth  required        pam_unix.so" >> $pam
                echo "account       required        pam_unix.so" >> $pam			
		    fi
			# FREENAS needs to copy files to /conf/base/etc folder to ensure that it survives restart
            case "`uname -i`" in
              FREENAS*)
                if test -d "/conf/base/etc/pam.d"; then
                    cp /etc/pam.d/nsd /conf/base/etc/pam.d/nsd
                fi
              ;;
            esac								
            ;;
          Darwin)
            cat /etc/pam.d/sudo | grep -v pam_securikey.so > /etc/pam.d/nsd
            ;;
          SunOS)
            # Defaults already handle what we need
            ;;
        esac
        if test $# -eq 0; then
            printf "\n"
            printf "Use: \"$LEXXDIR/start-server\" to start the software\n"
            printf "Use: \"$LEXXDIR/stop-server\" to stop the software\n"
            printf "\n"
        fi
        ;;
      10)
        case "`uname -r`" in
          5.6|5.7)
            "$LEXXDIR/etc/awjb-install" "$LEXXDIR"
            ;;
          *)
            "$LEXXDIR/etc/sgen-install" "$LEXXDIR"
            ;;
        esac
        ;;
      99)
        done="y"
        ;;
      *)
        ;;
    esac
done

exit $ret

# EOF $RCSfile: install.sh,v $
